﻿using Common;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http;
using TGood.Xian.LogManager;
using ThoughtWorks.QRCode.Codec;
using WxPayAPI;
using WxPayAPI.Models;
using wxWebAPI.Common;
using wxWebAPI.Filters;
using wxWebAPI.Models;

namespace WxPayAPI
{
    public partial class OrdersController : ApiController
    {
        #region 微信登录、支付、付款回调

        [HttpPost, HttpOptions]
        public JObject Login([FromBody]string code)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                string userToken = chargeLogic.Login(code);
                if (!string.IsNullOrEmpty(userToken))
                {
                    result.Add("code", 1);
                    result.Add("data", userToken);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("OrdersController - Login", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject WXMINAPay([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                string parameters = chargeLogic.WXMINAPay(param);
                if (!string.IsNullOrEmpty(parameters))
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - WXMINAPay - param=[{param.ToString()}]", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject WXAPPPay([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var parameters = chargeLogic.WXAPPPay(param);
                if (parameters != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(parameters));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - WXAPPPay - param=[{param.ToString()}]", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public string WXMINAPayNotify()
        {
            LogEvent.WebAPILoger.Debug("WXMINAPayNotify, WXMINAPayNotify进来了");
            string result = string.Empty;
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.WXMINAPayNotify();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("OrdersController - WXMINAPayNotify", ex);
            }

            return result;
        }

        [HttpPost, HttpOptions]
        public string WXAPPPayNotify()
        {
            string result = string.Empty;
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.WXAPPPayNotify();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("OrdersController - WXAPPPayNotify", ex);
            }

            return result;
        }

        #endregion

        #region 退款相关

        /// <summary>
        /// 申请退款完整业务流程逻辑
        /// </summary>
        /// <param name="transaction_id">微信订单号（优先使用）</param>
        /// <param name="out_trade_no">商户订单号</param>
        /// <param name="total_fee">订单总金额</param>
        /// <param name="refund_fee">退款金额</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public string Refund([FromBody] WXRefund refundParameter)
        {
            LogEvent.WebAPILoger.Debug("Refund 进来了");
            string result = string.Empty;
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.Refund(refundParameter);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - Refund", ex);
            }

            return result;
        }

        /// <summary>
        /// 退款查询完整业务流程逻辑
        /// </summary>
        /// <param name="transaction_id">微信退款单号（优先使用）</param>
        /// <param name="out_trade_no">商户退款单号</param>
        /// <param name="total_fee">微信订单号</param>
        /// <param name="refund_fee">商户订单号</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public string RefundQuery(string refund_id, string out_refund_no, string transaction_id, string out_trade_no)
        {
            return WxPayAPI.RefundQuery.Run(refund_id, out_refund_no, transaction_id, out_trade_no);
        }

        #endregion

        #region 充电相关
        /// <summary>
        /// 前端扫到没有|的内容时，调用的方法
        /// </summary>
        /// <param name="device"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetGunOfDevice(string device)
        {
            JObject result = new JObject();
            try
            {
                ChargeLogic chargeLogic = new ChargeLogic();
                List<string> gun = chargeLogic.GunOfDevice(device);
                if (gun.Any())
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(gun));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "查无数据");
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"GetGunOfDevice device{device}");
                result.Add("code", 0);
                result.Add("error", e.ToString());

            }
            return result;
        }



        /// <summary>
        /// 获取用户充电历史订单，返回小于minID的20条记录，首次请求minID不传或传0，后继传上次获取到的对象中最小的id
        /// </summary>
        /// <param name="userToken">用户标示</param>
        /// <param name="minID">最小ID</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetHistoryOrder(string userToken, int minID = 0)
        {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var chargeLogic = new ChargeLogic();
                IEnumerable<Order> historyOrderList = chargeLogic.GetHistoryOrder(userToken, minID);
                if (historyOrderList != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(historyOrderList));
                    //当前订单中最小id
                    result.Add("minid", historyOrderList.Any() ? historyOrderList.Last().id : 0);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryOrder userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }
        /// <summary>
        /// 查看月账单
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="month"></param>
        /// <returns></returns>
        [HttpGet,HttpOptions]
        public JObject GetMonthOrder(string userToken,string month = null,string map = null)
        {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var chargeLogic = new ChargeLogic();
                IEnumerable<dynamic> historyOrderList = chargeLogic.GetMonthOrder(userToken,month,map);
                if (historyOrderList != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(historyOrderList));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetMonthOrder userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;

        }


        /// <summary>
        /// 获取用户充电历史订单
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetHistoryOrder(string userToken, int year, int month)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<Order> historyOrderList = chargeLogic.GetHistoryOrder(userToken, year, month);
                if (historyOrderList != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(historyOrderList));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryOrder Error,userToken={userToken},year={year},month={month}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// /获取设备最近一次的充电订单
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetDeviceLastOrder(string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                Order lastOrder = chargeLogic.GetDeviceLastOrder(deviceCode);
                if (lastOrder != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(lastOrder));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetDeviceLastOrder deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据用户的openId获取活跃订单
        /// </summary>
        /// <param name="openId">用户的openId</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetActiveOrderByOpenId(string openId)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<Order> activeOrderList = chargeLogic.GetActiveOrderByOpenId(openId);
                if (activeOrderList != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(activeOrderList));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetActiveOrderByOpenId openId={openId}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据设备的deviceCode获取活跃订单
        /// </summary>
        /// <param name="deviceCode">设备编码</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetActiveOrderByDeviceCode(string deviceCode)
        {
            LogEvent.WebAPILoger.Info($"GetActiveOrderByDeviceCode deviceCode={deviceCode}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.GetActiveOrderByDeviceCode(deviceCode);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetActiveOrderByDeviceCode deviceCode={deviceCode}", ex);
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 获取用户充电最近的订单
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject NewGetCurrentOrder(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var chargeLogic = new ChargeLogic();
                Order currentOrder = chargeLogic.GetCurrentOrder(userToken, deviceCode);
                if (currentOrder != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(currentOrder));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryOrder.userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取用户充电最近的订单
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [System.Web.Http.HttpGet]
        [TokenFilter]
        public JObject GetCurrentOrder(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                Order currentOrder = chargeLogic.GetCurrentOrder(userToken);
                if (currentOrder != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(currentOrder));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryOrder.userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取充电实时数据
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject NewGetRealData(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var chargeLogic = new ChargeLogic();
                ChargeData chargeData = chargeLogic.GetRealData(userToken, deviceCode);

                if (chargeData != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(chargeData));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetRealData.userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }


        /// <summary>
        /// 获取充电实时数据
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [System.Web.Http.HttpGet]
        [TokenFilter]
        public JObject GetRealData(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                ChargeData chargeData = chargeLogic.GetRealData(userToken);

                if (chargeData != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(chargeData));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetRealData.userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }


        [HttpGet, HttpOptions]
        public JObject PostVinTimeInfo(string deviceCode,string startTime,string usertoken) {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(usertoken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                ChargeLogic chargelogic = new ChargeLogic();
                bool insertTime = chargelogic.PostVinTimeInfo(deviceCode,startTime,usertoken);
                if (insertTime == true)
                {
                    result.Add("code", 1);
                    result.Add("date", "success");
                }
                else {
                    result.Add("code", 0);
                    result.Add("date", "error");
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", "fail");
                result.Add("error", e.ToString());
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetVinTimeInfo(string usertoken)
        {
            ChargeLogic chargelogic = new ChargeLogic();
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                var openid = redis.HashGetString(RedisKey.Token, usertoken);

                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var data =  chargelogic.GetVinTimeInfo(usertoken);
                if (data != null)
                {
                    result.Add("code", 1);
                    result.Add("StartTime", JProperty.FromObject(data));
                }
                else {
                    result.Add("code", 0);
                    result.Add("data", "暂无预约数据");
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("error", e.ToString());
                LogEvent.WebAPILoger.Error(e);
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject DeleteVinTimeInfo(string userToken) {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                ChargeLogic cr = new ChargeLogic();
                bool data =  cr.DeleteVinTimeInfo(userToken);
                if (data == true)
                {
                    result.Add("code", 1);
                }
                else {
                    result.Add("code", 0);
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("error", e.ToString());
            }
            return result;
        }
        /// <summary>
        /// 发送停止充电请求
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject NewPostStop()
        {
           
            var userToken = HttpContext.Current.Request["userToken"];
            var deviceCode = HttpContext.Current.Request["deviceCode"];
            
            JObject result = new JObject();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
            {
                result.Add("code", -10);
                result.Add("error", "Token过期，请重新登陆");
                return result;
            }

            try
            {
                var chargeLogic = new ChargeLogic();
                var stopResult = chargeLogic.PostStop(userToken, deviceCode);

                if (stopResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "success");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PostStop - userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }


        /// <summary>
        /// 发送停止充电请求
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [System.Web.Http.HttpPost]
        [TokenFilter]
        public JObject PostStop([FromBody]string userToken)
        {
            LogEvent.WebAPILoger.Debug($"PostStop, userToken={userToken}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var stopResult = chargeLogic.PostStop(userToken);

                if (stopResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "此处可能返回一些其他信息");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PostStop - userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }


        /// <summary>
        /// 手动结束订单
        /// </summary>
        /// <param name="userToken"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject ManualCloseOrder([FromBody]HistoryOrder order)
        {
            LogEvent.WebAPILoger.Info($"ManualCloseOrder, order:{order.ObjectToString()}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var close = chargeLogic.ManualCloseOrder(order);
                if (close)
                {
                    result.Add("code", 1);
                    result.Add("data", "关闭订单成功");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "关闭订单失败");
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("ManualCloseOrder", ex);
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 获取设备状态
        /// </summary>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <returns>1 可用;0 其他</returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public object NewGetDeviceStatusByUserToken(string userToken)
        {
            JObject result = new JObject();
            List<ChargeState> gunStatus = new List<ChargeState>();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                //LogEvent.WebAPILoger.Debug($"GetDeviceStatusByUserToken 进入,userToken={userToken}");
                var chargeLogic = new ChargeLogic();
                gunStatus = chargeLogic.NewGetDeviceStatusByUserToken(userToken);
                if (gunStatus == null || gunStatus.Count == 0)
                {
                    result.Add("code", 0);
                    // result.Add("description", "当前用户有未结束订单");
                    result.Add("data", "无数据！");
                }
                //else if (gunStatus == (int)Common.EnumGunStatus.空闲 || gunStatus == (int)Common.EnumGunStatus.连接)
                //{
                //    result.Add("code", 1);
                //  result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                //}
                else
                {
                    result.Add("code", 1);
                    //result.Add("description", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                    result.Add("data", JProperty.FromObject(gunStatus));
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetDeviceStatus。userToken={userToken}.", ex);
                result.Add("code", 0);
                result.Add("data", ex.ToString());

            }
            return result;
        }

        /// <summary>
        /// 获取设备状态
        /// </summary>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <returns>1 可用;0 其他</returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetDeviceStatusByUserToken(string userToken)
        {
            JObject result = new JObject();
            try
            {
                //LogEvent.WebAPILoger.Debug($"GetDeviceStatusByUserToken 进入,userToken={userToken}");
                var chargeLogic = new ChargeLogic();
                var gunStatus = chargeLogic.GetDeviceStatusByUserToken(userToken);

                if (gunStatus == 99)
                {
                    result.Add("code", 99);
                    result.Add("data", "当前用户有未结束订单");
                }
                //else if (gunStatus == (int)Common.EnumGunStatus.空闲 || gunStatus == (int)Common.EnumGunStatus.连接)
                //{
                //    result.Add("code", 1);
                //   result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                //}
                else
                {
                    result.Add("code", 0);
                    result.Add("data", Enum.GetName(typeof(EnumGunStatus), gunStatus));

                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetDeviceStatus。userToken={userToken}.", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }


        #endregion

        #region 局站相关
        /// <summary>
        /// 获取周边局站信息
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="longitude"></param>
        /// <param name="latitude"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetStation(string userToken, double longitude, double latitude)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<dynamic> stations = chargeLogic.GetStation(userToken, longitude, latitude);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStation Error,userToken={userToken},longitude={longitude},latitude={latitude}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取局站明细信息
        /// </summary>
        /// <param name="stationCode"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStationDetail(string stationCode, string usertoken = null)
        {
            JObject result = new JObject();

            try
            {
                //默认小程序为领充 支付方式为微信
                EnumAppName app = EnumAppName.领充;
                EnumPayChannel payChannel = EnumPayChannel.微信;
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetStationDetail(stationCode, usertoken);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                    result.Add("paymentWay", (int)chargeLogic.GetPaymentMethod(stationCode, ref payChannel, ref app));
                    result.Add("app", (int)app);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStationDetail Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据code获取局站明细信息
        /// </summary>
        /// <param name="stationCode"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStationDetailByCode(string code, string usertoken = null)
        {
            JObject result = new JObject();

            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetStationDetailByCode(code, usertoken);
                if (stations != null && stations.Any())
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                    result.Add("paymentWay", (int)stations.FirstOrDefault().payChannel);
                    result.Add("app", (int)stations.FirstOrDefault().app);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStationDetailByDeviceCode Error,Code={code},usertoken={usertoken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据经纬度和半径获取范围内局站
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="radius">查找半径，单位km</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetNearStationSummary(double longitude, double latitude, double radius)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                Dictionary<string, StackExchange.Redis.GeoPosition> stations = chargeLogic.GetNearStationSummary(longitude, latitude, radius);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(stations.ToList()));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetNearStationSummary Error,longitude={longitude},latitude={latitude},radius={radius}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据条件查询符合条件的局站
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="radius">查找半径，单位km</param>
        /// <param name="stationType">局站类型，值按位从低到高依次代表为公共站、专用站、个人站，全选传0</param>
        /// <param name="chargeGunType">充电枪类型，值按位从低到高依次代表为交流慢充、交流快充、直流快充，全选传0</param>
        /// <param name="isParkingFree">是否免费停车，为null的话不判断该条件</param>
        /// <param name="isWorking">局站是否营业中，依据系统录入的营业时间</param>
        /// <param name="chargeStandardType">支持的充电接口国标类型，国标2011 = 1, 国标2015 = 2, 两者都有 = 3</param>
        /// <param name="auxiliaryPowerType">支持的充电辅源电压类型，辅源12V = 1, 辅源24V = 2, 两者都有 = 3</param>
        /// <returns></returns>
        [HttpGet,HttpOptions]
        public JObject GetNearStation(double longitude, double latitude, double radius
            , int stationType, int chargeGunType, bool? isParkingFree, bool? isWorking, string companyID = null, string usertoken = null,string appname = null
            , EnumChargeStandardType chargeStandardType = EnumChargeStandardType.两者都有
            , EnumAuxiliaryPowerType auxiliaryPowerType = EnumAuxiliaryPowerType.两者都有
            , EnumStationOrderBy orderby = EnumStationOrderBy.距离最近)
        {
            JObject result = new JObject();
            try
            {
                if (radius < 0)
                {
                    radius = 15;
                }
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetNearStation(appname,usertoken, longitude, latitude, radius
                    , stationType, chargeGunType, isParkingFree, isWorking, companyID, chargeStandardType, auxiliaryPowerType, orderby);
                if (stations != null)
                {
                    
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                var errorMsg = new StringBuilder();
                errorMsg.Append($"GetNearStation Error,longitude={longitude},latitude={latitude},radius={radius},");
                errorMsg.Append($"stationType={stationType},chargeGunType={chargeGunType},isParkingFree={isParkingFree},");
                errorMsg.Append($"isWorking={isWorking},chargeStandardType={chargeStandardType},auxiliaryPowerType={auxiliaryPowerType}");
                LogEvent.WebAPILoger.Error(errorMsg.ToString(), ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据经纬度和关键字获取范围内局站(默认半径100公里)
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="keyword">关键字</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStationByKeyword(double longitude, double latitude, string keyword,string appname = null,string companyid = null)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetStationByKeyword(longitude, latitude, keyword,appname,companyid);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(stations.ToList()));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetNearStationSummary Error,longitude={longitude},latitude={latitude},keyword={keyword}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取其他信息
        /// </summary>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetOtherInfo(string companyID = "")
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<string> infos = chargeLogic.GetOtherInfo(companyID);
                if (infos != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(infos));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("GetOtherInfo Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }




        #endregion

        #region 江汉出行相关

        /// <summary>
        /// 江汉租车调用的充电接口
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject PostStartCharge(string deviceCode)
        {
            //TODO:考虑，使用DES加密，密钥m双方约定好，deviceCode加密传输，
            //我方接受后解密，判断设备如连接，则启动充电，同时返回经纬度,
            //如不可用，返回枪具体状态和经纬度
            JObject result = new JObject();
            try
            {
                //var chargeLogic = new ChargeLogic();
                //IEnumerable<string> infos = chargeLogic.GetOtherInfo();
                //if (infos != null)
                //{
                //    result.Add("code", 1);
                //    result.Add("data", JProperty.FromObject(infos));
                //}
                //else
                //{
                //    result.Add("code", 0);
                //    result.Add("data", null);
                //}
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PostStartCharge Error,deviceCode={deviceCode}", ex);
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }
        #endregion

        #region Test

        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject SaveVIP(string userToken, string nickName)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                bool saveResult = chargeLogic.SaveVIP(userToken, nickName);
                if (saveResult)
                {
                    result.Add("code", 1);
                    result.Add("data", null);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - SaveVIP Error,userToken={userToken},nickName={nickName}", ex);
                result.Add("code", 0);
                result.Add("error", ex.ToString());

            }
            return result;
        }

        #endregion

        #region 支付宝登录、支付、付款回调

        [HttpPost,HttpOptions]
        public JObject ALILogin(string code)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                string userToken = chargeLogic.ALILogin(code, "");
                if (!string.IsNullOrEmpty(userToken))
                {
                    result.Add("code", 1);
                    result.Add("data", userToken);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("OrdersController - ALILogin Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject ALIMINAPay([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                string parameters = chargeLogic.ALIMINAPay(param);
                if (!string.IsNullOrEmpty(parameters))
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - ALIMINAPay Error,param={param.ToString()}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject ALIAPPPay([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                string parameters = chargeLogic.ALIAPPPay(param);
                if (!string.IsNullOrEmpty(parameters))
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - ALIAPPPay Error,param={param.ToString()}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public string ALIMINAPayNotify()
        {
            string result = string.Empty;
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.ALIMINAPayNotify();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - ALIMINAPayNotify", ex);
            }

            return result;
        }

        [HttpPost, HttpOptions]
        public string ALIAPPPayNotify()
        {
            string result = string.Empty;
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.ALIAPPPayNotify();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - ALIAPPPayNotify", ex);
            }

            return result;
        }

        #endregion   

        #region APP相关

        /// <summary>
        /// app版本检测
        /// </summary>
        /// <param name="version">当前版本号</param>
        /// <returns>code返回码，data最新版本信息，error错误信息</returns>
        [HttpGet, HttpOptions]
        public JObject CheckAPPVersion(int version)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                APPVersion checkResult = chargeLogic.CheckAPPVersion(version);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(checkResult));
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - CheckAPPVersion Error,version={version}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 发送短信验证码
        /// </summary>
        /// <param name="requestInfo">PhoneNum:目标手机号码,RequestTime:请求发送验证码时间,MD5:计算出的md5值</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject SendPIN([FromBody]PINInfo requestInfo)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                int sendResult = chargeLogic.SendPIN(requestInfo);
                switch (sendResult)
                {
                    case -1:
                        result.Add("code", -1);
                        result.Add("data", "sent more than maximum");
                        break;
                    case -2:
                        result.Add("code", -2);
                        result.Add("data", "sending interval is too short");
                        break;
                    case -3:
                        result.Add("code", -3);
                        result.Add("data", "md5 check is wrong");
                        break;
                    case 1:
                        result.Add("code", 1);
                        result.Add("data", "sending");
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - SendPIN Error,phoneNum={requestInfo.PhoneNum},RequestTime={requestInfo.RequestTime},MD5={requestInfo.MD5}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 检查校验码是否正确，并返回设备标识和用户标识
        /// </summary>
        /// <param name="phoneNumCode">key为手机号码，value为验证码</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject CheckPIN([FromBody]KeyValuePair<string, string> phoneNumCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                JObject checkResult = chargeLogic.CheckPIN(phoneNumCode);
                if (checkResult != null)
                {
                    result.Add("code", 1);
                    result.Add("data", checkResult);
                }
                else
                {
                    result.Add("code", 0);
                    //result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - CheckPIN Error,PhoneNum={phoneNumCode.Key},PIN={phoneNumCode.Value}。", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }

            return result;
        }

        /// <summary>
        /// APP登录
        /// </summary>
        /// <param name="login">DeviceID为移动设备唯一标识</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject APPLogin([FromBody]APP_Login login)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                JObject loginResult = chargeLogic.APPLogin(login);
                if (loginResult != null)
                {
                    result.Add("code", 1);
                    result.Add("data", loginResult);
                }
                else
                {
                    result.Add("code", 0);
                    //result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - APPLogin Error,login.PhoneNum={login.PhoneNum},login.DeviceID={login.DeviceID},login.MD5={login.MD5}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }

            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetShareInfo()
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var info = chargeLogic.GetShareInfo();

                if (info != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(info));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetShareInfo Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 根据电话号码获取用户信息，包括账户余额
        /// </summary>
        /// <param name="userToken">手机号码</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetAPPUser(string userToken)
        {
            JObject result = new JObject();
           
            try
            {
                var chargeLogic = new ChargeLogic();
                if (chargeLogic.checkToken(userToken))
                {
                    var user = chargeLogic.GetAPPUser(userToken);
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(user));
                }
                else
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，或传入的token为空");
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetAPPUser Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 查看用户余额
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetAPPUserBalance(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                int? userBalance = chargeLogic.GetAPPUserBalance(userToken);
                if (userBalance.HasValue)
                {
                    result.Add("code", 1);
                    result.Add("data", userBalance.Value);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", 0);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetAPPUserBalance Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 更新用户信息
        /// </summary>
        /// <param name="user">用户信息</param>
        /// <param name="userToken">访问令牌</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        [TokenFilter]
        public JObject UpdateAPPUser([FromBody] APP_User user, string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                APP_User updateResult = chargeLogic.UpdateAPPUser(user, userToken);
                if (updateResult == null)
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
                else
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(updateResult));
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - UpdateAPPUser Error,userToken={userToken},user.loginid={user.loginid}," +
                                            $"user.nickname={user.nickname},user.sex={user.sex},user.birthday={user.birthday}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 更新用户头像
        /// </summary>
        /// <param name="userProfile">头像信息，现为Base64编码字符串</param>
        /// <param name="userToken">访问令牌</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        [TokenFilter]
        public JObject UploadAPPUserProfile([FromBody] UserProfile userProfile, string userToken)
        {
            JObject result = new JObject();
            try
            {
                string Imagepath = string.Empty;
                var chargeLogic = new ChargeLogic();
                Boolean uploadResult = chargeLogic.UploadAPPUserProfile(userProfile, userToken,ref Imagepath);

                if (uploadResult)
                {
                    result.Add("code", 1);
                    result.Add("path", Imagepath);
                }
                else
                {
                    result.Add("code", 0);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - UploadAPPUserProfile Error,userToken={userToken}, userProfile={userProfile}", ex);
                result.RemoveAll();
                result.Add("code", 0);
            }
            return result;
        }

        /// <summary>
        /// 获取用户历史充值记录，返回小于minID的20条记录，首次请求minID不传或传0，后继传上次获取到的对象中最小的id
        /// </summary>
        /// <param name="userToken">用户标示</param>
        /// <param name="minID">最小ID</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetHistoryRecharge(string userToken, int minID = 0)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<Recharge> historyRechargeList = chargeLogic.GetHistoryRecharge(userToken, minID);
                if (historyRechargeList != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(historyRechargeList));
                    //当前订单中最小id
                    result.Add("minid", historyRechargeList.Any() ? historyRechargeList.Last().id : 0);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetHistoryRecharge userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        #endregion

        #region 发票相关
        /// <summary>
        /// 用户获取可以打印发票的订单
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="minID"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public List<JObject> GetNoInvoiceHistoryOrder(string userToken, int minID = 0)
        {
            JObject result = new JObject();
            var chargeLogic = new ChargeLogic();
            List<JObject> historyOrderList = chargeLogic.GetNoInvoiceHistoryOrder(userToken, minID);
            return historyOrderList;
        }

        /// <summary>
        /// 保存用户地址及企业抬头信息
        /// </summary>
        /// <param name="address"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject PostAddressInfo([FromBody]InvoicePurchaserAddress address)
        {

            JObject result = new JObject();
            ChargeLogic chargeLogic = new ChargeLogic();
            try
            {
                var insertResult = chargeLogic.InsertAddressIntoDb(address);
                if (insertResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "成功！");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "失败！");
                }
            }
            catch (Exception ex)
            {
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }
        /// <summary>
        /// 获取用户的地址及企业抬头信息
        /// </summary>
        /// <param name="UserToken"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetAddressInfo(string UserToken)
        {
            JObject result = new JObject();
            ChargeLogic chargeLogic = new ChargeLogic();
            try
            {
                var info = chargeLogic.GetAddressInfo(UserToken);
                if (info.Any())
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(info));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "查无数据");
                }

            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error("GetAddRessInfo 获取用户地址抬头信息失败");
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", e.ToString());
            }
            return result;
        }
        /// <summary>
        /// 用户发送需要定制发票的所有信息
        /// </summary>
        /// <param name="invoice"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject PostInvoice([FromBody]Invoice invoice)//Invoice invoice
        {
            JObject result = new JObject();
            ChargeLogic chargeLogic = new ChargeLogic();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                var openid = redis.HashGetString(RedisKey.Token, invoice.UserToken);

                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                var insertResult = chargeLogic.InsertInvoiceIntoDb(invoice);
                if (insertResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "成功！");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "失败！");
                }
            }
            catch (Exception ex)
            {
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;

        }

        /// <summary>
        /// 商户查看需要开发票的信息
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="type"></param>
        /// <param name="AppID"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetInvoiceState(string userToken)
        {
            JObject result = new JObject();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                var openid = redis.HashGetString(RedisKey.Token, userToken);

                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                ChargeLogic chargelogic = new ChargeLogic();
                var invoice = chargelogic.GetInvoiceState(userToken);
                if (invoice.Count() > 0)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(invoice));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "查无数据");
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("error", e.ToString());
                LogEvent.WebAPILoger.Error("获取Invoice异常！",e);
            }


            return result;
        }


        [HttpGet, HttpOptions]
        public JObject GetInvoiceAddress(string usertoken)
        {
            JObject result = new JObject();
            ChargeLogic chargeLogic = new ChargeLogic();
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                string openid = redis.HashGetString(RedisKey.Token, usertoken);
                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期");
                    return result;
                }
                var info = chargeLogic.GetInvoiceAddress(usertoken);
                if (info.Any())
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(info));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "查无数据");
                }

            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error("GetAddRessInfo 获取用户地址抬头信息失败");
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", e.ToString());
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject CancleInvoice(DateTime Addtime,string usertoken,string companyID) {

            JObject result = new JObject();
            try
            {
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(usertoken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                ChargeLogic chargelogic = new ChargeLogic();
                bool cancle = chargelogic.CancleInvoice(Addtime, usertoken,companyID);
                if (cancle == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "撤销成功！");
                }
                else {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", $"CancleInvoice Addtime={Addtime} == {e.ToString()}");
            }
            return result;
        }
        #endregion
        
    }
}
